#######################   PSQF 6249 FA & SEM Example 9 CFA using R   #########################

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=4) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

if (!require("semTools")) install.packages("semTools")
library(semTools) # lavaan add-ons (multiple imputation, moderation, omega)

# Other packages not used here but that are likely useful

#if (!require("semPlot")) install.packages("semPlot")
#library(semPlot) # lavaan add-ons to make diagrams

#if (!require("MplusAutomation")) install.packages("MplusAutomation")
#library(lavaan) # To use Mplus within R

#################################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6249/DriverScan/"
filename = "driverscanSEM.xlsx"
setwd(dir=filesave)

# Import item-level data
driverscanSEM = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
driverscanSEM = as.data.frame(driverscanSEM)

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example9_SEM_Output.txt"))

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data = myData)

#################################################################################################
# CFA Measurement Model 1 for Visual Impairment
# Factor Variance estimated, first loading=1, Factor Mean = 0, all intercepts estimated

VisualSyntax = "vision =~ 1*far + lncs15 + lncs3 + lncs6 + lncs12 + lncs18"
Visual = cfa(model=VisualSyntax, data=driverscanSEM, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Visual, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
print("Get omega reliability using model-implied variances and unequal loadings")
compRelSEM(object=Visual, obs.var=FALSE, tau.eq=FALSE)
# Get model-implied means, variances, and covariances
fitted(object=Visual)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=Visual, type="raw"); resid(object=Visual, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=Visual, type="normalized")

txtComment(" ") # insert blank space in output
  

#################################################################################################
# CFA Measurement Model 2 for Driving Impairment
# Factor Variance estimated, first loading=1, Factor Mean = 0, all intercepts estimated

DrivingSyntax = "driving =~ 1*crash + da_task + lane + stop + speed + time
                 speed ~~ time"
Driving = cfa(model=DrivingSyntax, data=driverscanSEM, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Driving, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
print("Get omega reliability using model-implied variances and unequal loadings")
compRelSEM(object=Driving, obs.var=FALSE, tau.eq=FALSE)
# Get model-implied means, variances, and covariances
fitted(object=Driving)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=Driving, type="raw"); resid(object=Driving, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=Driving, type="normalized")

txtComment(" ") # insert blank space in output  


#################################################################################################
# CFA Measurement Model 3 for Attentional Impairment
# Factor Variance estimated, first loading=1, Factor Mean = 0, all intercepts estimated

AttnSyntax = "attn =~ 1*lnda + lnsa + Dscan"
Attn = cfa(model=AttnSyntax, data=driverscanSEM, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=Attn, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
print("Get omega reliability using model-implied variances and unequal loadings")
compRelSEM(object=Attn, obs.var=FALSE, tau.eq=FALSE)

txtComment(" ") # insert blank space in output  


#################################################################################################
# Structural Equation Model 4a (saturated, latent variable main effects only)
# Factor Variance estimated, first loading=1, Factor Mean = 0, all intercepts estimated

SEM4aSyntax = "
# Measurement models
  vision =~ 1*far + lncs3 + lncs6 + lncs12 + lncs15 + lncs18
  driving =~ 1*crash + da_task + lane + stop + speed + time; speed ~~ time
  attn =~ 1*lnda + lnsa + Dscan
  pspeed =~ 1*lnps; lnps ~~ 0*lnps
# Structural model regressions
  vision  ~ (Age1)*age75
  attn    ~ (Age2)*age75 + (Vis1)*vision
  pspeed  ~ (Age3)*age75 + (Vis2)*vision
  driving ~ (Age4)*age75 + (Vis3)*vision + (Speed1)*pspeed + (Attn1)*attn
# Structural model disturbance covariances
  attn ~~ pspeed
# Indirect effects
  AgeVis   := Age1*Vis3
  AgeSpeed := Age3*Speed1
  AgeAttn  := Age2*Attn1
"
SEM4a = cfa(model=SEM4aSyntax, data=driverscanSEM, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=SEM4a, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
print("Get model-predicted latent variable correlations")
lavInspect(object=SEM4a, what="cor.lv")
# Get model-implied means, variances, and covariances
fitted(object=SEM4a)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=SEM4a, type="raw"); resid(object=Driving, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=SEM4a, type="normalized")

txtComment(" ") # insert blank space in output  


#################################################################################################
# Structural Equation Model 4b (no age or vision --> driving, latent variable main effects only)
# Factor Variance estimated, first loading=1, Factor Mean = 0, all intercepts estimated

SEM4bSyntax = "
# Measurement models
  vision =~ 1*far + lncs3 + lncs6 + lncs12 + lncs15 + lncs18
  driving =~ 1*crash + da_task + lane + stop + speed + time; speed ~~ time
  attn =~ 1*lnda + lnsa + Dscan
  pspeed =~ 1*lnps; lnps ~~ 0*lnps
# Structural model regressions
  vision  ~ (Age1)*age75
  attn    ~ (Age2)*age75 + (Vis1)*vision
  pspeed  ~ (Age3)*age75 + (Vis2)*vision
  driving ~ (0)*age75 + (0)*vision + (Speed1)*pspeed + (Attn1)*attn
# Structural model disturbance covariances
  attn ~~ pspeed
"
SEM4b = cfa(model=SEM4bSyntax, data=driverscanSEM, estimator="MLR", mimic="mplus", std.lv=FALSE)
# Print solution: get fit, get effect size, STDYX solution
summary(object=SEM4b, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
print("LRT for comparison against saturated structural model")
anova(SEM4b, SEM4a)
print("Get model-predicted latent variable correlations")
lavInspect(object=SEM4b, what="cor.lv")
# Get model-implied means, variances, and covariances
fitted(object=SEM4b)
# Get residual (discrepancy) info for local misfit in covariance and correlation form
resid(object=SEM4b, type="raw"); resid(object=Driving, type="cor")
# Get normalized residuals (like z-scores for discrepancies)
resid(object=SEM4b, type="normalized")

txtComment(" ") # insert blank space in output  


#################################################################################################
# SEM 5a and 5b (with latent variable interactions) does not appear to be possible in lavaan
# SEM 5c is same as path model pretending factor scores are perfectly reliable



#################################################################################################
# Close external file of saved results
txtStop()


